import MagicString from "magic-string";
import { rollup, watch, } from "rollup";
import { join } from "path/posix";
import { createHash } from "crypto";
import { existsSync, mkdirSync, readFileSync } from "fs";
function getAssetHash(content) {
    return createHash("sha256").update(content).digest("hex").slice(0, 8);
}
let isBuild;
let config;
export const ENTRY_FILE_MARKER = "entry_file";
const plugins_config = new WeakMap();
const internal_plugin_config = new Map();
internal_plugin_config.set("vite:import-analysis", []);
const plugin = {
    name: "bundle:helper",
    configResolved(resolvedConfig) {
        if (config)
            return;
        isBuild = resolvedConfig.command === "build";
        config = resolvedConfig;
    },
    transform(code, id) {
        if (!isBuild && id.endsWith(ENTRY_FILE_MARKER)) {
            const nCode = new MagicString(code).prepend(`import '/@vite/env'\n`);
            return {
                code: nCode.toString(),
                map: nCode.generateMap({ hires: false }),
            };
        }
    },
};
export default function bundleHelper() {
    return plugin;
}
const watchers = [];
export async function bundle(pluginContext, entry, ctx, result_type = "module", inline = false, watch_files = false, static_file = false) {
    if (isBuild && watch_files) {
        throw new Error("At build you can not watch files!");
    }
    if (!isBuild && inline) {
        throw new Error("You only can inline files at build!");
    }
    if (inline && result_type === "module") {
        throw new Error("You only can inline files at build that are of type iife not module!");
    }
    if (entry.includes("?"))
        entry = entry + "&" + ENTRY_FILE_MARKER;
    else
        entry = entry + "?" + ENTRY_FILE_MARKER;
    if (result_type === "module") {
        if (isBuild) {
            const file = pluginContext.emitFile({
                type: "chunk",
                id: entry,
            });
            return `import.meta.ROLLUP_FILE_URL_${file}`;
        }
        else {
            return "'/@id/" + entry + "'";
        }
    }
    if (result_type === "iife") {
        const rollup_config = {
            input: entry,
            plugins: [
                ...config.plugins.filter((v) => inContext(v, ctx)),
                // LAST PLUGIN IS CUSTOM!
                {
                    name: "extra-file-loader",
                    load(id) {
                        if (id.includes("?")) {
                            return readFileSync(id.split("?")[0], "utf-8");
                        }
                    },
                },
            ],
        };
        if (watch_files) {
            const cacheDir = join(config.cacheDir, ".bundle.iife");
            if (!existsSync(cacheDir))
                mkdirSync(cacheDir);
            const fileName = static_file
                ? static_file
                : getAssetHash(Buffer.from(entry + ctx)) + ".iife.js";
            const outFile = join(cacheDir, fileName);
            const watcher = watch({ ...rollup_config, output: { file: outFile } });
            watchers.push(watcher);
            return new Promise((res, req) => {
                let first = true;
                watcher.on("event", async (event) => {
                    if (event.code === "ERROR" && first) {
                        first = false;
                        req(event.error);
                    }
                    if (event.code === "BUNDLE_END") {
                        await event.result.generate({
                            format: "iife",
                            sourcemap: "inline",
                        });
                        if (first) {
                            first = false;
                            pluginContext.addWatchFile(outFile);
                            res(JSON.stringify(outFile.slice(config.root.length)));
                        }
                    }
                });
            });
        }
        else {
            const bundle = await rollup(rollup_config);
            try {
                const { output } = await bundle.generate({
                    format: "iife",
                    sourcemap: config.build.sourcemap,
                });
                await bundle.close();
                const code = output[0].code;
                const content = Buffer.from(code);
                if (inline) {
                    const base64 = content.toString("base64");
                    const blob = `new Blob([atob("${base64}")], { type: 'text/javascript;charset=utf-8' })`;
                    return `(window.URL || window.webkitURL).createObjectURL(${blob})`;
                }
                if (!static_file) {
                    const pathSplit = entry.split("?")[0].split(/\/|\\/g);
                    const fileSplit = pathSplit[pathSplit.length - 1].split(".");
                    const basename = fileSplit.slice(0, fileSplit.length - 1).join(".");
                    const contentHash = getAssetHash(content);
                    const fileName = join(config.build.assetsDir, `${basename}.${contentHash}.js`);
                    // get real URL variable
                    return `'__VITE_ASSET__${pluginContext.emitFile({
                        fileName,
                        type: "asset",
                        source: code,
                    })}__'`;
                }
                else {
                    return `'__VITE_ASSET__${pluginContext.emitFile({
                        fileName: static_file,
                        type: "asset",
                        source: code,
                    })}__'`;
                }
            }
            catch (ex) {
                await bundle.close();
                throw ex;
            }
        }
    }
    throw new Error("Non valid result_type!");
}
export function setPluginContext(plugin, ctx, overwrite = true) {
    if (overwrite || !plugins_config.has(plugin)) {
        plugins_config.set(plugin, ctx);
    }
    else {
        plugins_config.set(plugin, plugins_config.get(plugin).concat(ctx));
    }
}
function inContext(plugin, ctx) {
    if (!plugins_config.has(plugin) && !internal_plugin_config.has(plugin.name))
        return true;
    if (plugins_config.has(plugin)) {
        return plugins_config.get(plugin).includes(ctx);
    }
    if (internal_plugin_config.has(plugin.name)) {
        return (internal_plugin_config.get(plugin.name).includes(ctx) ||
            internal_plugin_config.get(plugin.name).includes("*"));
    }
}
export function inlineURLRevoke(varName) {
    return `(window.URL || window.webkitURL).revokeObjectURL(${varName});`;
}
//# sourceMappingURL=index.js.map